package eu.clarin.cmdi.mdservice.action;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;

import javax.xml.transform.stream.StreamSource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import eu.clarin.cmdi.mdservice.internal.Admin;
import eu.clarin.cmdi.mdservice.model.Termset;

/**
 * Acts as a Struts 2 controller 
 * serves the requests regarding DataCategories
 * it passes them further (Proxy) to the Data Category Registry.
 *   
 * @author vronk 
 *
 */

public class DCRProxyAction extends GenericProxyAction {

	private String proxy_key = "dcregistry";
	private static Logger log = Logger.getLogger("DCRProxyAction");
	
	private String dcrid;
	
	public static String ALL = "all";
	public static String DATCAT = "datcat";
	public static String DATCATS = "datcats";
	
	private static final long serialVersionUID = 1L;
	
	
	public String getDcrid() {
		return dcrid;
	}

	public void setDcrid(String dcrid) {
		this.dcrid = dcrid;
	}

	public Termset getDCR() {		
		String dcr_id="";
		if (getQ()!=null && !getQ().equals("")) dcr_id = getQ();
		// override with dcrid-param
		if (getDcrid()!=null && !getDcrid().equals("")) dcr_id = getDcrid();			

		Termset res = (Termset) Termset.getTermset(dcr_id);
		if (res==null)		Admin.notifyUser("getDCR(): no termset found for:" + dcr_id);
		//else
			//Admin.notifyUser("getDCR():" + res.getAttr("id") + ":" + res.getAttr("url") );
		
		return res;
	}
	
	public String getDCRAttr(String key) {	
		Termset dcr = getDCR();
		String val=""; 
		if (dcr!=null) val= dcr.getAttr(key);
		 return val;
	}
	
	@Override
	public String getProxyKey() {
		return proxy_key;
	}
	 
	public String getFullFormat() {
		Termset dcr = getDCR();
		String srcformat="", srctype=""; 
		if (dcr!=null){ 
				srcformat = dcr.getAttr("format");
				srctype = dcr.getAttr("type");
		} else { 
			 srcformat = ALL;
			 srctype = "datcat";
		}
			return srctype + "-" + srcformat  + "2" + getFormat();		
	}
	
	public URL getBaseURL() throws MalformedURLException {
		
		if (base_url == null) {
			base_url = new URL(getBaseURI());
		}
		
		return base_url;
	}
	
	public String getBaseURI() {		
		return getDCRAttr("url");
	}
	
	/**
	 * uses base_url and (optionally) id to form a url
	 * no id => ?list all datacategories?
	 * @return the URL to data category xml format
	 * @throws Exception 
	 */
	public URL getTargetRequest() throws IOException {

		URL targetURL = null;
	
		//if (getActionkey().equals(DATCATS)) {
		if ((getQ()!=null && !getQ().equals(""))) {
			//targetURL =new URL(getBaseURI() + "?dcif-mode=all&workingLanguage=" + getLang()) ;
			targetURL =new URL(getBaseURI() + "?dcif-mode=all&workingLanguage=" + getLang()) ;
		}
		/*
			if (getQ()==null || getQ().equals("")) {			
					// get all available datcat-sets from terms_setup.xml
				
				targetURL =getBaseURL();
			} else {
				targetURL =new URL( getBaseURL(), "/" + getQ());
			}
			*/
//		Admin.notifyUser("DCRProxy.targetURL:" + targetURL);

	    return targetURL;
	}
	
	public InputStream getSourceStream() throws IOException {

		InputStream resultStream = null; 
		// if no arguments  - get the local-config file 
		// FIXME: although this should be competence of TermsProxyAction
		if ((getQ().equals(ALL) || getQ()==null || getQ().equals("")) && (getActionkey().equals(DATCATS))) {
			log.debug("serving terms.file:" + Admin.getConfig().getProperty("terms.file"));
			resultStream = this.getClass().getClassLoader().getResourceAsStream(Admin.getConfig().getProperty("terms.file"));
		} else {
		
		/* need to set Request-Property: Accept to xml 
		 * for to get dcif/xml from DCR-server by means of content negotiation */
		URLConnection urlConnection = getTargetRequest().openConnection();
		
		if (getDCRAttr("format").equals("dcif")) {		
			urlConnection.setRequestProperty("Accept", "application/dcif+xml");
			log.debug("setting header Accept:" + urlConnection.getRequestProperty("Accept"));
		}
        //urlConnection.setRequestProperty("Accept-Language", getLang());
        
        //
        	resultStream = urlConnection.getInputStream();
		}
        return resultStream;    
    
	}

}