package eu.clarin.cmdi.mdservice.proxy;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;

/**
 * Proxy to the Metadata Repository.  
 * Responds to requests (collections, model, recordset) 
 * by dispatching the requests to appropriate internal methods and filling the inputStream with the result
 * 
 * It is possible to switch target-repository dynamically  (on every request). I.e. the client explicitly selects one of the repositories.
 *  
 * @author vronk
 *
 */
public class MDRepoProxy extends RepoProxy {

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("MDRepoProxy");
	
	private String proxy_key = "mdrepository";
	
	/**
	 * Override of the basic method provided by the super-class 
	 */	
	@Override	
	public String getBaseURI() {		
		String uri = WorkspaceProfile.getRepositoryPath(getSourceAction().getRepository());		
		return uri;
	}
	
	@Override
	public String getProxyKey() {
		return proxy_key;
	}
	
	/**
	 * A mapping between the actionkeys in the request and the operation-parameter expected by the MDRepository 
	 */
	private final static HashMap<String,String> urls = new HashMap<String,String>();
	 static
	 {	 urls.put("collection", "?operation=getCollections&collection=");
	  	 urls.put("model", "?operation=queryModel&q=");
	  	 urls.put("values", "?operation=scanIndex&q=");
	 	 urls.put("recordset", "?operation=searchRetrieve&query=");
	 	 urls.put("record", "?operation=searchRetrieve&query=");
	 } 
 
	  
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws Exception 
	 */
	 @Override
	public URL getTargetRequest() throws IOException {
        // URL targetURL =new URL( base_url, compname + ".xml");
		 return getBaseURL();
		/*
		log.debug("MDRPx.getQ:" + getSquery() + " and (" +  getQ() + ")");
		log.debug("MDRPA.getActionkey:" + getActionkey());		
		
		Query query = new Query(getSquery(), getQ(),getActionkey());

	// check if the query could get parsed
		if (query.isStatus(Query.PARSEERROR)) {
			log.debug("MDRPA.query.PARSEERROR:" + query.getMsg());
			// pass this bad news to the client
			setUserMsg(query.getMsg());
			return null;
		} else {
			query.setMaxdepth(getMaxdepth());		
			query.setCollection(getCollection());
			query.setColumns(getColumns());
			query.setMaximumItems(getMaximumItems());
			query.setStartItem(getStartItem());
			query.setOptions(getOptions());
			query.setSort(getSort());
		
			URL targetURL = null;
			targetURL =new URL( getBaseURL(), urls.get(getActionkey()) + query.toURLParam() );
			Admin.notifyUser("MDRPA.targetURL.query.toURLParam:" + query.toURLParam());
			Admin.notifyUser("MDRPA.targetURL:" + targetURL);
	        return targetURL;
		}
		*/
		 
	}

	@Override
	public InputStream getSourceStream() throws IOException, NoStylesheetException {
		
		/*if (getCommand() == null){
			try {
				return getSourcePz2();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} 
			*/
		return super.getSourceStream();
	}
	
}