<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:my="myFunctions"
 exclude-result-prefixes="my">

<!-- 
<purpose>generic templates for MDService2</purpose>
<history>
	<change on="2010-03-19" type="created" by="vr">based on 3RMA/suki/dc4_commons.xsl</change>			
	<change on="2010-03-23" type="changed" by="vr">really started rework - added html wrapper, ...</change>			
	<change on="2010-12-29" type="changed" by="vr">extracted function into: cmd_functions.xsl</change>
</history>

-->

<xsl:include href="cmd_functions.xsl"/>
<xsl:include href="html_snippets.xsl"/>

<xsl:param name="src_file" />	
<!--  FIXME: he? -->	
<xsl:param name="root_uri" select="$src_file" />		

<!-- common starting point for all stylesheet; cares for unified html-envelope
and passes back to the individual stylesheets for the content (via template: continue-root) -->
<xsl:template match="/">
	<!--	<xsl:message>root_document-uri:<xsl:value-of select="$root_uri" /></xsl:message>-->		
	  	<xsl:choose>    		
    	 	<xsl:when test="contains($format,'htmlpage')" >
    	 		<xsl:call-template name="html"/>    
    	 	</xsl:when>
    	 	<xsl:when test="contains($format,'htmlsimple')" >
    	 		<xsl:call-template name="htmlsimple"/>    
    	 	</xsl:when>
    	 	<xsl:otherwise>    	 		
    	 		<xsl:call-template name="continue-root"/>    
    	 	</xsl:otherwise>
    	 </xsl:choose>
</xsl:template>


<xsl:template name="html">

	<html xmlns="http://www.w3.org/1999/xhtml" >
	<head>
		<xsl:call-template name="html-head"></xsl:call-template>
	   	<xsl:call-template name="callback-header"/> 
	</head>
	<body>
		
		<xsl:call-template name="page-header" />	
	
		<h1><xsl:value-of select="$title"/></h1>
				
		<xsl:call-template name="continue-root"/>
		    
	</body>
	</html>
</xsl:template>

<!-- a html-envelope for a simple (noscript) view -->
<xsl:template name="htmlsimple">
	<html xmlns="http://www.w3.org/1999/xhtml" > 
	<head>
 		<title><xsl:value-of select="$title"/></title>
 		<link href="{$base_dir}/style/cmds-ui.css" type="text/css" rel="stylesheet"></link>

   		<!-- <xsl:call-template name="callback-header"/> -->   		 	
	</head>
	
	<xsl:call-template name="page-header" />
		
	<body>
		<!-- <h1><xsl:value-of select="$title"/></h1> -->		
			<xsl:call-template name="continue-root"/>    
	</body>
	</html>
</xsl:template>

<xsl:template name="callback-header">

</xsl:template>

<!-- 
<xsl:template name="continue-root">
</xsl:template>
 -->
    
<!--  generic templates -->
<xsl:template name="attr-detail-div" >
		<div class="detail">
			<xsl:for-each select="@*" >
				<div class="cmds-elem-prop"><span class="label"><xsl:value-of select="name()" />: </span>
					<span class="value"><xsl:value-of select="." /></span></div>
			</xsl:for-each>
		</div>
</xsl:template>

<xsl:template name="format-field" >
	<xsl:param name="elems" />
  					<xsl:choose>
  						 <xsl:when test="$elems/*" > 					
			  					<xsl:apply-templates select="$elems" mode="format-xmlelem" >	
			  					</xsl:apply-templates>		  					
		  				</xsl:when>
		  				<xsl:when test="count($elems) &gt; 1" >
		  					<ul>
			  					<xsl:for-each select="$elems" >		  					
			  						<li><xsl:call-template name="format-value" /></li>
			  					</xsl:for-each>
		  					</ul>
		  				</xsl:when>
		  				<xsl:otherwise>
		  					<xsl:call-template name="format-value">
		  						<xsl:with-param name="value" select="$elems" />
		  					</xsl:call-template>		  					
		  				</xsl:otherwise>  			
			  		</xsl:choose>
					 
</xsl:template>

<xsl:template name="format-value" >
<xsl:param name="value" select="."/>
	<!-- cnt_value:<xsl:value-of select="count($value)" />  -->
  					<xsl:choose>
  						 <xsl:when test="starts-with($value[1], 'http:') " >
								<a target="_blank" class="external" href="{$value}"><xsl:value-of select="$value"></xsl:value-of></a>
						</xsl:when>
						<xsl:otherwise>
								<xsl:value-of select="$value" />
						</xsl:otherwise>
					</xsl:choose>
</xsl:template>  				

<!-- copied from mdinst2view.xsl
	generic html-view for xml-elements  
-->
<xsl:template match="*" mode="format-xmlelem" >
	
	<xsl:if test="exists(.//text()) or @*" >
		<xsl:variable name="has_text" >
			<xsl:choose>
				<xsl:when test="normalize-space(text()[1])='Unspecified'">unspecified</xsl:when>
				<xsl:when test="not(normalize-space(string-join(text(),''))='')">text</xsl:when>				
				<xsl:otherwise>empty</xsl:otherwise>
			</xsl:choose>			
		</xsl:variable>
		<xsl:variable name="has_children" select="if(*) then 'has-children' else ''"  />
			
		<div class="cmds-xmlelem {$has_children} value-{$has_text}" >		
				<span class="label"><xsl:value-of select="name()" />:</span>
				<span class="value"><xsl:call-template name="format-value" > 
										<xsl:with-param name="value" select="text()[.!='']" />
									</xsl:call-template>
				</span>
			<xsl:apply-templates select="*" mode="format-xmlelem"/>
			<xsl:if test="@*">
				<div class="attributes">
					<xsl:apply-templates select="@*" mode="format-attr"/>
				</div>
			</xsl:if>
		</div>	
	</xsl:if>
</xsl:template>	

<xsl:template match="@*" mode="format-attr">
	<span class="label">@<xsl:value-of select="name()" />: </span><span class="value"><xsl:call-template name="format-value" /><!--<xsl:value-of select="." /> --></span>; 
</xsl:template>	

<xsl:template name="getName" >
		<xsl:value-of select="(.//*[lower-case(name()) = $name_element])[1]" /> 
</xsl:template>

<xsl:template name="getContext" >
<!--	<div class="is-part-of collections" ><xsl:value-of select=".//IsPartOf" /> </div>-->
	<xsl:apply-templates select=".//IsPartOfList" />
</xsl:template>

	
<xsl:template match="IsPartOfList" >
	<div class="ispartof-list"><span class="label">in: </span>
		<xsl:apply-templates select=".//IsPartOf" />
	</div>
</xsl:template>	

<xsl:template match="IsPartOf" >
	<a class="internal" href="{concat($default_prefix, my:encodePID(.))}"><xsl:value-of select="." /></a> >
</xsl:template>	


</xsl:stylesheet>